package com.xsnb.auth.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.xsnb.auth.feign.AuthFeignSystem;
import com.xsnb.auth.service.AuthService;
import com.xsnb.common.constants.JwtConstants;
import com.xsnb.common.constants.RabbitMQConstants;
import com.xsnb.common.constants.TokenConstants;
import com.xsnb.common.domain.User;
import com.xsnb.common.domain.request.RequestUser;
import com.xsnb.common.domain.response.RespJwt;
import com.xsnb.common.result.Result;
import com.xsnb.common.utils.JwtUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import java.util.regex.Pattern;

@Service
@Log4j2
public class AuthServiceImpl implements AuthService {

    @Autowired
    AuthFeignSystem authFeignSystem;

    @Autowired
    RedisTemplate<String, String> redisTemplate;
    @Autowired
    RabbitTemplate rabbitTemplate;


    @Override
    public Result<RespJwt> login(RequestUser requestUser) {
        String phone = requestUser.getPhone();
        String captcha = requestUser.getCaptcha();
        if (null == phone || null == captcha) {
            return Result.error("联系管理员");
        }
        if ("".equals(phone) || "".equals(captcha)) {

            return Result.error("请输入手机号+密码");
        }
        Result<User> byphone = authFeignSystem.byphone(phone);
        User data = byphone.getData();
        if (null == data) {
            return Result.error("先注册手机号,在登录");
        }
        //从redsi拿验证码进行比对
        //判断名字为"PHONE_"+phone的key是否存在
        String s = redisTemplate.opsForValue().get(phone);
        log.info("拿到的:{}",s);
        if (null == s || !s.equals(captcha)) {
            return Result.error("验证码错误");
        }
        HashMap<String, Object> map = new HashMap<>();
        String userkey = UUID.randomUUID().toString().replaceAll("-", "");
        map.put(JwtConstants.USER_KEY, userkey);
        map.put(JwtConstants.DETAILS_USER_ID, data.getId());
        String token = JwtUtils.createToken(map);
        redisTemplate.opsForValue().set(
                TokenConstants.LOGIN_TOKEN_KEY + userkey,
                JSONObject.toJSONString(data),
                TokenConstants.EXPIRATION,
                TimeUnit.MINUTES
        );
        RespJwt respJwt = new RespJwt();
        respJwt.setToken(token);
        respJwt.setEidTime("720MIN");
        log.info("登录成功,token:{}", token);
        return Result.success(respJwt);
    }

    /*
     * 前台每次请求都会来到这里进行解析user
     *
     * */
    @Override
    public Result<User> userinfo(HttpServletRequest request) {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSONObject.parseObject(s, User.class);
        /*获取用户信息*/
        log.info("userInfo获取的用户信息是:{}", user);
        return Result.success(user);
    }

    /**
     * @param user
     * @return
     * @dis 短信发送
     */
    @Override
    public Result getPhoneCode(User user) {
        String phone = user.getPhone() == null ? "18906604007" : user.getPhone();

        log.info("短信发送的手机号 :{}", phone);
        Result<User> byphone = authFeignSystem.byphone(phone);
        User data = byphone.getData();
        if (null == data) {
            return Result.error("先注册手机号,在登录");
        }
        Pattern compile = Pattern.compile("^(?:(?:\\+|00)86)?1\\d{10}$");
        if (!compile.matcher(phone).matches()) {
            return Result.error("手机号格式错误");
        }
        rabbitTemplate.convertAndSend(RabbitMQConstants.SEND_SMS_QUEUs, phone, message -> {
            message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
            log.info("rabbit异步发送完毕 :{}", phone);
            return message;
        });

        return Result.success("开始发送短信");
    }

}
