package com.xsnb.system.rabbitMQ;

import cn.hutool.core.util.RandomUtil;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.dysmsapi20170525.models.SendSmsResponseBody;
import com.rabbitmq.client.Channel;
import com.xsnb.common.constants.RabbitMQConstants;
import com.xsnb.common.utils.TelSmsUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.HashMap;

@Component
@Log4j2
public class MessageConsumerService {

    @Autowired
    private RedisTemplate<String,String> redisTemplate;


    @RabbitListener(queuesToDeclare = {@Queue(RabbitMQConstants.SEND_SMS_QUEUs)})
    public void SmsController(String phone, Message message, Channel channel){
        long s = System.currentTimeMillis();
        String messageId = message.getMessageProperties().getMessageId();
        log.info("短信消费队列开始消费  消息内容：{}，开始执行",phone);
        try {
            Long count = redisTemplate.opsForSet().add(RabbitMQConstants.SEND_SMS_QUEUs, messageId);
            if(count==1){
                String code = RandomUtil.randomNumbers(4);
                String jsons = TelSmsUtils.sendSms(phone, new HashMap<String, String>() {{
                    put("code",code);
                }});
                SendSmsResponseBody sendSmsResponseBody = JSONObject.parseObject(jsons, SendSmsResponseBody.class);
                if(!"OK".equals(sendSmsResponseBody.getCode())){
                    TelSmsUtils.sendSms(phone,new HashMap<String, String>(){{
                        put("code",code);
                    }});
                    channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
                    redisTemplate.opsForValue().set(phone,JSONObject.toJSONString(code));
                    log.info("短信消费队列消费成功 用户：{}，耗时：{}",phone,System.currentTimeMillis()-s);
                }
            }else {
                log.info("短信重复消费 内容：{}",phone);
            }
        } catch (Exception e) {
            log.info("消息队列消费失败 异常信息 用户：{}， 内容：{}",phone,e.getMessage());
            try {
                channel.basicReject(message.getMessageProperties().getDeliveryTag(),true);
            } catch (Exception exception) {
                log.error("消息队列消费失败  回退信息 用户：{},内容：{}",phone,exception.getMessage());
            }
        }
    }
}
