package com.xsnb.system.utils;

import com.xsnb.system.domain.Attachment;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.core.io.FileSystemResource;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;

import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeUtility;
import java.io.File;
import java.util.List;

/**
 * FileName: EmailUtils
 * Author: 张海宁
 * Date:   2024/3/4 13:53
 * Description:邮箱工具类
 * History:
 * <author>      <time>     <version>     <desc>
 * 作者姓名       修改时间      版本号         描述
 */
@Slf4j
public class EmailUtils {


    public static Integer sendMailAttachment(JavaMailSender mailSender,
                                             String fromEmail,
                                             String toEmail,
                                             String subject,
                                             String content,
                                             Attachment attachment) {
        MimeMessage message = mailSender.createMimeMessage();
        Integer result = 1;
        try {
            MimeMessageHelper helper = new MimeMessageHelper(message, true, "UTF-8");
            helper.setFrom(fromEmail);
            helper.setTo(toEmail);
            helper.setSubject(subject);
            helper.setText(content, true);

            if (attachment != null && attachment.getFileName() != null && attachment.getFilePath() != null) {
                File destinationFile = new File(attachment.getFilePath());
                FileSystemResource fileResource = new FileSystemResource(destinationFile);
                helper.addAttachment(destinationFile.getName(), fileResource);
            }

            mailSender.send(message);
            log.info("邮件发送成功,收件人地址:" + toEmail);
        } catch (Exception e) {
            result = 0;
            log.error("邮件发送出现异常,收件人地址:" + toEmail, e);
        }
        return result;
    }

    public static Integer sendMailAttachments(JavaMailSender mailSender, String fromEmail, String toEmail,
                                              String subject, String content, List<Attachment> attachments) {
        System.setProperty("mail.mime.splitlongparameters", "false");
        MimeMessage message = mailSender.createMimeMessage();
        try {
            MimeMessageHelper helper = new MimeMessageHelper(message, true, "UTF-8");
            helper.setFrom(fromEmail);
            helper.setTo(toEmail);
            helper.setSubject(subject);
            helper.setText(content, true);
            if (!ObjectUtils.isEmpty(attachments)) {
                attachments.forEach((attachment) -> {
                    try {
                        FileSystemResource file = new FileSystemResource(attachment.getFilePath());
                        helper.addAttachment(MimeUtility.
                                encodeWord(attachment.getFileName(),
                                        "UTF-8",
                                        "B"),
                                file);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                });
            }
            mailSender.send(message);
            log.info("邮件发送成功,收件人地址:" + toEmail);
            return 1;
        } catch (Exception e) {
            log.error("邮件发送出现异常,收件人地址:" + toEmail, e);
        }
        return 0;
    }
}
